@@ -56,6 +56,9 @@ _deprecated_id_re = re.compile(r'^\w+$')
 def valid_id(s):
     if valid_id_re.match(s):
         return True
+    elif s == '_method':
+        #ignore this one small hack to help pages use HTTP verbs
+        return True
     elif _deprecated_id_re.match(s):
         warn("The id %s will no longer be supported since it doesn't conform "
              "to the W3C Spec: http://www.w3.org/TR/xhtml1/#C_8" % s,
@@ -64,7 +67,6 @@ def valid_id(s):
     else:
         return False
 
-        
 serial_generator = count()
 
 
@@ -160,7 +162,8 @@ class Widget(view.Renderable):
         'css_classes': 'A list with extra css classes for the widget.'
         }
     template = None
-    engine_name = 'toscawidgets'    
+    engine_name = 'toscawidgets'
+    available_engines = ['toscawidgets']
     css = []
     javascript = []
     css_classes = []
@@ -251,9 +254,17 @@ class Widget(view.Renderable):
         likely. 
         """
         obj = view.Renderable.__new__(cls,id,parent,children,**kw)
-        obj.__dict__.update(
-            v for v in kw.iteritems() if not v[0].startswith('_')
-            )
+        # The previous version of this used the __dict__ attribute to update
+        # things, but that doesn't fire any fset properties, and can actually
+        # lose data if a property is set, the following will not behave like
+        # that
+        for k,v in kw.iteritems():
+            if not k.startswith('_'):
+                try:
+                    setattr(obj, k, v)
+                except AttributeError, e:
+                    #skip setting the value of a read only property
+                    pass
         obj.orig_kw = kw.copy()
         
         if id is not None and not valid_id(id):
@@ -538,9 +549,9 @@ class Widget(view.Renderable):
         """Returns the default value for the widget. If the default is a funtion
         that it can be called without arguments it will be called on each
         render to retrieve a value"""
-        try:
+        if callable_wo_args(self.default):
             value = self.default()
-        except TypeError:
+        else:
             value = self.default
         return value
 
@@ -571,8 +582,12 @@ class Widget(view.Renderable):
         d['c'] = d['children'] = self.children
         d = make_bunch(d)
         self.update_params(d)
+        s = set([d['css_class'],])
+        classes = d['css_classes']
+        for item in classes:
+            s.add(item)
         # Compute the final css_class string
-        d['css_class']= ' '.join(set([d['css_class'] or ''] + d['css_classes']))
+        d['css_class']= ' '.join(s)
         # reset the getters here so update_params has a chance to alter
         # the arguments to children and the value
         d['args_for'] = self._get_child_args_getter(d['child_args'])
@@ -633,11 +648,9 @@ class Widget(view.Renderable):
                     attr = copy(attr)
                 # Variables that are callable with no args are automatically
                 # called here
-                elif not isinstance(attr, Widget):
-                    try:
-                        attr = attr()
-                    except (TypeError, AttributeError):
-                        pass
+                elif not isinstance(attr, Widget) and callable_wo_args(attr):
+                    log.debug("Autocalling param '%s'", k)
+                    attr = attr()
             d[k] = attr
 
 
